"use strict";
// Load our dependencies
var stringify_1 = require("../common/stringify");
// Define our context Karma constructor
var ContextKarma = (function () {
    function ContextKarma(callParentKarmaMethod) {
        var _this = this;
        this.callParentKarmaMethod = callParentKarmaMethod;
        this.hasError = false;
        // Define our loggers
        // DEV: These are intentionally repeated in client and context
        this.log = function (type, args) {
            var values = [];
            for (var i = 0; i < args.length; i++) {
                values.push(_this.stringify(args[i], 3));
            }
            _this.info({ log: values.join(', '), type: type });
        };
        this.stringify = stringify_1.stringify;
        // Define our proxy error handler
        // DEV: We require one in our context to track `hasError`
        this.error = function () {
            var _arguments = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _arguments[_i - 0] = arguments[_i];
            }
            _this.hasError = true;
            _this.callParentKarmaMethod('error', [].slice.call(_arguments));
            return false;
        };
        // all files loaded, let's start the execution
        this.loaded = function () {
            // has error -> cancel
            if (!_this.hasError) {
                _this.start(_this.config);
            }
            // remove reference to child iframe
            _this.start = _this.UNIMPLEMENTED_START;
        };
        // supposed to be overriden by the context
        // TODO(vojta): support multiple callbacks (queue)
        this.start = this.UNIMPLEMENTED_START;
        // Define bindings for context window
        this.setupContext = function (contextWindow) {
            // If we clear the context after every run and we already had an error
            //   then stop now. Otherwise, carry on.
            if (_this.config.clearContext && _this.hasError) {
                return;
            }
            // Perform window level bindings
            // DEV: We return `self.error` since we want to `return false` to ignore errors
            contextWindow.onerror = function () {
                var _arguments = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    _arguments[_i - 0] = arguments[_i];
                }
                return _this.error.apply(_this, _arguments);
            };
            // DEV: We must defined a function since we don't want to pass the event object
            contextWindow.onbeforeunload = function (e, b) {
                _this.callParentKarmaMethod('onbeforeunload', []);
            };
            contextWindow.dump = function () {
                var _arguments = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    _arguments[_i - 0] = arguments[_i];
                }
                return _this.log('dump', _arguments);
            };
            contextWindow.alert = function (msg) { return _this.log('alert', [msg]); };
            // If we want to overload our console, then do it
            var getConsole = function (currentWindow) { return currentWindow.console || {
                log: function () { },
                info: function () { },
                warn: function () { },
                error: function () { },
                debug: function () { }
            }; };
            if (_this.config.captureConsole) {
                // patch the console
                var localConsole = contextWindow.console = getConsole(contextWindow);
                var logMethods = ['log', 'info', 'warn', 'error', 'debug'];
                var patchConsoleMethod = function (method) {
                    var orig = localConsole[method];
                    if (!orig) {
                        return;
                    }
                    localConsole[method] = function () {
                        var _arguments = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            _arguments[_i - 0] = arguments[_i];
                        }
                        _this.log(method, _arguments);
                        return Function.prototype.apply.call(orig, localConsole, _arguments);
                    };
                };
                for (var i = 0; i < logMethods.length; i++) {
                    patchConsoleMethod(logMethods[i]);
                }
            }
        };
        // Define proxy methods
        // DEV: This is a closured `for` loop (same as a `forEach`) for IE support
        var proxyMethods = ['complete', 'info', 'result'];
        for (var i = 0; i < proxyMethods.length; i++) {
            (function (methodName) {
                _this[methodName] = function boundProxyMethod() {
                    callParentKarmaMethod(methodName, [].slice.call(arguments));
                };
            })(proxyMethods[i]);
        }
    }
    // Define our start handler
    ContextKarma.prototype.UNIMPLEMENTED_START = function () {
        var _ = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            _[_i - 0] = arguments[_i];
        }
        this.error('You need to include some adapter that implements __karma__.start method!');
    };
    ContextKarma.prototype.info = function (param) {
    };
    /** Define call/proxy methods */
    ContextKarma.getDirectCallParentKarmaMethod = function (parentWindow) {
        return function directCallParentKarmaMethod(method, args) {
            // If the method doesn't exist, then error out
            if (!parentWindow.karma[method]) {
                parentWindow.karma.error('Expected Karma method "' + method + '" to exist but it doesn\'t');
                return;
            }
            // Otherwise, run our method
            parentWindow.karma[method].apply(parentWindow.karma, args);
        };
    };
    ContextKarma.getPostMessageCallParentKarmaMethod = function (parentWindow) {
        return function postMessageCallParentKarmaMethod(method, args) {
            parentWindow.postMessage({ __karmaMethod: method, __karmaArguments: args }, window.location.origin);
        };
    };
    return ContextKarma;
}());
exports.ContextKarma = ContextKarma;
//# sourceMappingURL=karma.js.map