"use strict";
var browser_1 = require("./browser");
var EXECUTING = browser_1.Browser.STATE_EXECUTING;
var browser_result_1 = require("./browser_result");
var BrowserCollection = (function () {
    function BrowserCollection(emitter, browsers) {
        if (browsers === void 0) { browsers = []; }
        var _this = this;
        this.emitter = emitter;
        this.browsers = browsers;
        this.add = function (browser) {
            _this.browsers.push(browser);
            _this.emitter.emit('browsers_change', _this);
        };
    }
    BrowserCollection.prototype.remove = function (browser) {
        var index = this.browsers.indexOf(browser);
        if (index === -1) {
            return false;
        }
        this.browsers.splice(index, 1);
        this.emitter.emit('browsers_change', this);
        return true;
    };
    BrowserCollection.prototype.getById = function (browserId) {
        for (var i = 0; i < this.browsers.length; i++) {
            if (this.browsers[i].id === browserId) {
                return this.browsers[i];
            }
        }
        return null;
    };
    BrowserCollection.prototype.setAllToExecuting = function () {
        this.browsers.forEach(function (browser) { return browser.state = EXECUTING; });
        this.emitter.emit('browsers_change', this);
    };
    BrowserCollection.prototype.areAllReady = function (nonReadyList) {
        nonReadyList = nonReadyList || [];
        this.browsers.forEach(function (browser) {
            if (!browser.isReady()) {
                nonReadyList.push(browser);
            }
        });
        return nonReadyList.length === 0;
    };
    BrowserCollection.prototype.serialize = function () {
        return this.browsers.map(function (browser) { return browser.serialize(); });
    };
    BrowserCollection.prototype.getResults = function () {
        var results = this.browsers.reduce(function (previous, current) {
            previous.success += current.lastResult.success;
            previous.failed += current.lastResult.failed;
            previous.error = previous.error || current.lastResult.error;
            previous.disconnected = previous.disconnected || current.lastResult.disconnected;
            return previous;
        }, { success: 0, failed: 0, error: false, disconnected: false, exitCode: 0 });
        // compute exit status code
        results.exitCode = results.failed || results.error || results.disconnected ? 1 : 0;
        return results;
    };
    // TODO(vojta): can we remove this? (we clear the results per browser in onBrowserStart)
    BrowserCollection.prototype.clearResults = function () {
        this.browsers.forEach(function (browser) { return browser.lastResult = new browser_result_1.Result(); });
    };
    BrowserCollection.prototype.clone = function () {
        return new BrowserCollection(this.emitter, this.browsers.slice());
    };
    // Array APIs
    BrowserCollection.prototype.map = function (callback, context) {
        return this.browsers.map(callback, context);
    };
    BrowserCollection.prototype.forEach = function (callback, context) {
        return this.browsers.forEach(callback, context);
    };
    Object.defineProperty(BrowserCollection.prototype, "length", {
        // this.length
        get: function () {
            return this.browsers.length;
        },
        enumerable: true,
        configurable: true
    });
    return BrowserCollection;
}());
exports.BrowserCollection = BrowserCollection;
BrowserCollection.$inject = ['emitter'];
//# sourceMappingURL=browser_collection.js.map