"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var events_1 = require("events");
var helper = require("./helper");
var bindAllEvents = function (object, context) {
    context = context || this;
    var bindMethod = function (method) {
        context.on(helper.camelToSnake(method.substr(2)), function () {
            var args = Array.prototype.slice.call(arguments, 0);
            args.push(context);
            object[method].apply(object, args);
        });
    };
    for (var method in object) {
        if (helper.isFunction(object[method]) && method.substr(0, 2) === 'on') {
            bindMethod(method);
        }
    }
};
exports.bufferEvents = function (emitter, eventsToBuffer) {
    var listeners = [];
    var eventsToReply = [];
    var genericListener = function () {
        eventsToReply.push(Array.prototype.slice.call(arguments));
    };
    eventsToBuffer.forEach(function (eventName) {
        var listener = genericListener.bind(null, eventName);
        listeners.push(listener);
        emitter.on(eventName, listener);
    });
    return function () {
        if (!eventsToReply) {
            return;
        }
        // remove all buffering listeners
        listeners.forEach(function (listener, i) {
            emitter.removeListener(eventsToBuffer[i], listener);
        });
        // reply
        eventsToReply.forEach(function (args) {
            events_1.EventEmitter.prototype.emit.apply(emitter, args);
        });
        // free-up
        listeners = eventsToReply = null;
    };
};
// TODO(vojta): log.debug all events
var KarmaEventEmitter = (function (_super) {
    __extends(KarmaEventEmitter, _super);
    function KarmaEventEmitter() {
        var _this = _super.call(this) || this;
        _this.bind = bindAllEvents;
        _this.emitAsync = function (name) {
            // TODO(vojta): allow passing args
            // TODO(vojta): ignore/throw if listener call done() multiple times
            var pending = _this.listeners(name).length;
            var deferred = helper.defer();
            var done = function () {
                if (!--pending) {
                    deferred.resolve();
                }
            };
            _this.emit(name, done);
            if (!pending) {
                deferred.resolve();
            }
            return deferred.promise;
        };
        return _this;
    }
    return KarmaEventEmitter;
}(events_1.EventEmitter));
exports.KarmaEventEmitter = KarmaEventEmitter;
// PUBLISH
exports.bindAll = bindAllEvents;
//# sourceMappingURL=events.js.map