"use strict";
var logger_1 = require("./logger");
var log = logger_1.create();
var Executor = (function () {
    function Executor(capturedBrowsers, config, emitter) {
        var _this = this;
        this.capturedBrowsers = capturedBrowsers;
        this.config = config;
        this.emitter = emitter;
        this.executionScheduled = false;
        this.pendingCount = 0;
        this.schedule = function () {
            var nonReady = [];
            if (!_this.capturedBrowsers.length) {
                log.warn('No captured browser, open %s//%s:%s%s', _this.config.protocol, _this.config.hostname, _this.config.port, _this.config.urlRoot);
                return false;
            }
            if (_this.capturedBrowsers.areAllReady(nonReady)) {
                log.debug('All browsers are ready, executing');
                log.debug('Captured %s browsers', _this.capturedBrowsers.length);
                _this.executionScheduled = false;
                _this.capturedBrowsers.clearResults();
                _this.capturedBrowsers.setAllToExecuting();
                _this.pendingCount = _this.capturedBrowsers.length;
                _this.runningBrowsers = _this.capturedBrowsers.clone();
                _this.emitter.emit('run_start', _this.runningBrowsers);
                _this.socketIoSockets.emit('execute', _this.config.client);
                return true;
            }
            log.info('Delaying execution, these browsers are not ready: ' + nonReady.join(', '));
            _this.executionScheduled = true;
            return false;
        };
        this.onRunComplete = function () {
            if (_this.executionScheduled) {
                _this.schedule();
            }
        };
        this.onBrowserComplete = function () {
            _this.pendingCount--;
            if (!_this.pendingCount) {
                // Ensure run_complete is emitted in the next tick
                // so it is never emitted before browser_complete
                setTimeout(function () {
                    _this.emitter.emit('run_complete', _this.runningBrowsers, _this.runningBrowsers.getResults());
                }, 0);
            }
        };
        // bind all the events
        emitter.bind(this);
    }
    return Executor;
}());
exports.Executor = Executor;
//# sourceMappingURL=executor.js.map