"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var events_1 = require("events");
var StateMachine = (function (_super) {
    __extends(StateMachine, _super);
    function StateMachine(rli, colors) {
        var _this = _super.call(this) || this;
        _this.rli = rli;
        _this.colors = colors;
        return _this;
    }
    StateMachine.prototype.showPrompt = function () {
        this.rli.write(this.colors.ANSWER);
        this.rli.prompt();
    };
    StateMachine.prototype.onKeypress = function (key) {
        if (!this.currentOptions || !key) {
            return;
        }
        if (key.name === 'tab' || key.name === 'right' || key.name === 'down') {
            this.suggestNextOption();
        }
        else if (key.name === 'left' || key.name === 'up') {
            this.currentOptionsPointer = this.currentOptionsPointer + this.currentOptions.length - 2;
            this.suggestNextOption();
        }
        if (!key.ctrl && !key.meta && key.name !== 'enter' && key.name !== 'return') {
            key.name = 'escape';
        }
    };
    StateMachine.prototype.suggestNextOption = function () {
        if (!this.currentOptions) {
            return;
        }
        this.currentOptionsPointer = (this.currentOptionsPointer + 1) % this.currentOptions.length;
        this.rli._deleteLineLeft();
        this.rli._deleteLineRight();
        this.rli.write(this.currentOptions[this.currentOptionsPointer]);
    };
    StateMachine.prototype.kill = function () {
        this.currentOptions = null;
        this.currentQuestionId = null;
        this.rli.write('\n' + this.colors.RESET + '\n');
        this.rli.close();
    };
    StateMachine.prototype.onLine = function (line) {
        if (this.currentQuestionId) {
            this.rli.write(this.colors.RESET);
            line = line.trim().replace(this.colors.ANSWER, '').replace(this.colors.RESET, '');
            if (this.currentOptions) {
                this.currentOptionsPointer = this.currentOptions.indexOf(line);
                if (this.currentOptionsPointer === -1) {
                    return;
                }
            }
            if (line === '') {
                line = null;
            }
            if (this.currentQuestion.boolean) {
                line = (line === 'yes' || line === 'true' || line === 'on');
            }
            if (line !== null && this.currentQuestion.validate) {
                this.currentQuestion.validate(line);
            }
            if (this.currentQuestion.multiple) {
                this.answers[this.currentQuestionId] = this.answers[this.currentQuestionId] || [];
                if (line !== null) {
                    this.answers[this.currentQuestionId].push(line);
                    this.showPrompt();
                    if (this.currentOptions) {
                        this.currentOptions.splice(this.currentOptionsPointer, 1);
                        this.currentOptionsPointer = -1;
                    }
                }
                else {
                    this.nextQuestion();
                }
            }
            else {
                this.answers[this.currentQuestionId] = line;
                this.nextQuestion();
            }
        }
    };
    StateMachine.prototype.nextQuestion = function () {
        this.currentQuestion = this.questions.shift();
        while (this.currentQuestion && this.currentQuestion.condition && !this.currentQuestion.condition(this.answers)) {
            this.currentQuestion = this.questions.shift();
        }
        this.emit('next_question', this.currentQuestion);
        if (this.currentQuestion) {
            this.currentQuestionId = null;
            this.rli.write('\n' + this.colors.question(this.currentQuestion.question) + '\n');
            this.rli.write(this.currentQuestion.hint + '\n');
            this.showPrompt();
            this.currentOptions = this.currentQuestion.options || null;
            this.currentOptionsPointer = -1;
            this.currentQuestionId = this.currentQuestion.id;
            this.suggestNextOption();
        }
        else {
            this.currentQuestionId = null;
            this.currentOptions = null;
            // end
            this.kill();
            this.done(this.answers);
        }
    };
    StateMachine.prototype.process = function (_questions, _done) {
        this.questions = _questions;
        this.answers = {};
        this.done = _done;
        this.nextQuestion();
    };
    return StateMachine;
}(events_1.EventEmitter));
exports.StateMachine = StateMachine;
//# sourceMappingURL=state_machine.js.map