"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var events_1 = require("../events");
var events_2 = require("events");
var Promise = require("bluebird");
var log = require('../logger').create('launcher');
var helper = require("../helper");
var BEING_CAPTURED = 1;
var CAPTURED = 2;
var BEING_KILLED = 3;
var FINISHED = 4;
var RESTARTING = 5;
var BEING_FORCE_KILLED = 6;
/**
 * Base launcher that any custom launcher extends.
 */
var BaseLauncher = (function (_super) {
    __extends(BaseLauncher, _super);
    function BaseLauncher(id, emitter) {
        var _this = _super.call(this) || this;
        _this.id = id;
        _this.emitter = emitter;
        _this.state = null;
        _this.error = null;
        _this.kill = function () {
            // Already killed, or being killed.
            if (_this.killingPromise) {
                return _this.killingPromise;
            }
            _this.killingPromise = _this.emitAsync('kill').then(function () {
                _this.state = FINISHED;
            });
            _this.state = BEING_KILLED;
            return _this.killingPromise;
        };
        _this.forceKill = function () {
            _this.kill();
            _this.state = BEING_FORCE_KILLED;
            return _this.killingPromise;
        };
        _this.restart = function () {
            if (_this.state === BEING_FORCE_KILLED) {
                return;
            }
            if (!_this.killingPromise) {
                _this.killingPromise = _this.emitAsync('kill');
            }
            _this.killingPromise.then(function () {
                if (_this.state === BEING_FORCE_KILLED) {
                    _this.state = FINISHED;
                }
                else {
                    _this.killingPromise = null;
                    log.debug('Restarting %s', _this.name);
                    _this.start(_this.previousUrl);
                }
            });
            _this.state = RESTARTING;
        };
        _this.markCaptured = function () {
            if (_this.state === BEING_CAPTURED) {
                _this.state = CAPTURED;
            }
        };
        _this.isCaptured = function () {
            return _this.state === CAPTURED;
        };
        _this._done = function (error) {
            _this.killingPromise = _this.killingPromise || Promise.resolve();
            _this.error = _this.error || error;
            _this.emit('done');
            if (_this.error && _this.state !== BEING_FORCE_KILLED && _this.state !== RESTARTING) {
                _this.emitter.emit('browser_process_failure', _this);
            }
            _this.state = FINISHED;
        };
        _this.STATE_BEING_CAPTURED = BEING_CAPTURED;
        _this.STATE_CAPTURED = CAPTURED;
        _this.STATE_BEING_KILLED = BEING_KILLED;
        _this.STATE_FINISHED = FINISHED;
        _this.STATE_RESTARTING = RESTARTING;
        _this.STATE_BEING_FORCE_KILLED = BEING_FORCE_KILLED;
        if (_this.start) {
            return _this;
        }
        // TODO(vojta): figure out how to do inheritance with DI
        Object.keys(events_2.EventEmitter.prototype).forEach(function (method) {
            this[method] = events_2.EventEmitter.prototype[method];
        }, _this);
        _this.start = function (url) {
            _this.previousUrl = url;
            _this.error = null;
            _this.state = BEING_CAPTURED;
            _this.emit('start', url + '?id=' + _this.id + (helper.isDefined(_this.displayName) ? '&displayName=' + encodeURIComponent(_this.displayName) : ''));
        };
        return _this;
    }
    BaseLauncher.prototype.toString = function () {
        return this.name;
    };
    BaseLauncher.decoratorFactory = function (id, emitter) {
        return function (launcher) {
            BaseLauncher.call(launcher, id, emitter);
        };
    };
    return BaseLauncher;
}(events_1.KarmaEventEmitter));
exports.BaseLauncher = BaseLauncher;
//# sourceMappingURL=base.js.map