"use strict";
var log = require('../logger').create('launcher');
var RetryLauncher = (function () {
    function RetryLauncher(retryLimit) {
        var _this = this;
        this._retryLimit = retryLimit;
        this.on('done', function () {
            if (!_this.error) {
                return;
            }
            if (_this._retryLimit > 0) {
                var attempt = retryLimit - _this._retryLimit + 1;
                log.info('Trying to start %s again (%d/%d).', _this.name, attempt, retryLimit);
                _this.restart();
                _this._retryLimit--;
            }
            else if (_this._retryLimit === 0) {
                log.error('%s failed %d times (%s). Giving up.', _this.name, retryLimit, _this.error);
            }
            else {
                log.debug('%s failed (%s). Not restarting.', _this.name, _this.error);
            }
        });
    }
    RetryLauncher.decoratorFactory = function (retryLimit) {
        return function (launcher) {
            RetryLauncher.call(launcher, retryLimit);
        };
    };
    return RetryLauncher;
}());
exports.RetryLauncher = RetryLauncher;
RetryLauncher.decoratorFactory.$inject = ['config.retryLimit'];
//# sourceMappingURL=retry.js.map